///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2012 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		TF	Dec 27, 2012	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.ccl;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.ccl.TdbCclCommand;
import com.tietoenator.trip.jxp.ccl.TdbHistoryDetail;
import com.tietoenator.trip.jxp.ccl.TdbKernelWindow;
import com.tietoenator.trip.jxp.ccl.TdbTerm;
import com.tietoenator.trip.jxp.ccl.TdbTermList;
import com.tietoenator.trip.jxp.ccl.TdbTermTree;
import com.tietoenator.trip.jxp.ccl.TdbTreeEntry;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.session.TdbSession;
import java.util.ArrayList;

/**
 * Shows how to use on-demand term lists. Requires TRIPsystem 7.0 or
 * later in order to work.
 */
public class TermLists
{

    private TdbCclCommand m_command;
    private TdbSession m_session;
    private ArrayList<TdbTermList> termLists;

    /**
     * Constructor
     *
     * @throws TdbException
     */
    public TermLists()
            throws TdbException
    {
        // Create a session on a physical server (no grid support)
        m_session = SessionFactory.newInstance("**** Concurrent Term List Example ****", false);
        termLists = new ArrayList<TdbTermList>();
    }

    public void setup()
            throws TdbException
    {
        // Show what we are connected to.
        System.out.println();
        System.out.println("Connected to TRIP " + m_session.getVersion());
        System.out.println();

        // Create the CCL command interpreter.
        m_command = new TdbCclCommand(m_session);
        
        // Declare that we want to use on-demand term lists without timeout.
        m_command.setUseOnDemandTermLists(true);
        m_command.setTermListTimeout(0);
    }

    private int selectTermList(Console stdin) throws TdbException
    {
        int listnr = 0;

        if (termLists.size() < 1)
        {
            System.out.println("There are no term lists available!");
        } 
        else
        {
            String cmd = stdin.readln("List  1 - " + Integer.toString(termLists.size()), "1", false);
            if (cmd != null)
            {
                try
                {
                    listnr = Integer.parseInt(cmd);
                } catch (NumberFormatException ignore)
                {
                    listnr = 0;
                }

                if (listnr < 1 || listnr > termLists.size())
                {
                    System.out.println("Invalid list number!");
                    listnr = 0;
                }
            }
        }
        return listnr;
    }

    public boolean process(Console stdin) throws TdbException
    {
        int listnr;
        boolean finished = false;

        System.out.println();
        System.out.println("Please select:");
        System.out.println(" [1] Exit");
        System.out.println(" [2] Execute CCL command");
        System.out.println(" [3] Show available term lists");
        System.out.println(" [4] Retrieve term list values");
        System.out.println(" [5] Release a term list");
        System.out.println(" [6] "+ (m_command.getUseOnDemandTermLists()?"Disable":"Enable") +" the on-demand term list feature");
        System.out.println(" [7] Set on-demand term list timeout");
        
        System.out.println();

        String cmd = null;
        int cmdnr = 0;
        do
        {
            while (cmd == null)
            {
                cmd = stdin.readln("Select 1-6 ", "2", false);
            }

            try
            {
                cmdnr = Integer.parseInt(cmd);
            } 
            catch (NumberFormatException ignore)
            {
                cmdnr = 0;
            }
        } 
        while (cmdnr < 1 || cmdnr > 7);

        switch (cmdnr)
        {
            case 1:
                finished = true;
                break;

            case 2:
                cmd = stdin.readln("CCL> ", null, false);
                finished = execCcl(cmd);
                break;

            case 3:
                if (termLists.isEmpty())
                    System.out.println("There are no term lists available!");
                
                for (int i = 0; i < termLists.size(); i++)
                {
                    TdbTermList list = termLists.get(i);
                    System.out.println("#" + Integer.toString(i + 1) + "\t" + 
                            Integer.toString(list.size()) + " terms\t"+
                            list.getCommand());
                }
                break;

            case 4:
                listnr = selectTermList(stdin);
                if (listnr > 0)
                {
                    TdbTermList list = termLists.get(listnr - 1);
                    if (list.isValid() == false)
                    {
                        System.out.println("Term list #"+Integer.toString(listnr)+" is no longer valid!");
                    }
                    else
                    {
                        System.out.println("" + list.size() + " terms");

                        int termnr = 1;
                        for (TdbTerm term : list)
                        {
                            System.out.println("T=" + termnr++ + "\t<" + term.getRecordCount() + ">\t" + term.getTerm());
                        }

                        // There is a small chance that the term list timed out between
                        // our list.isValid() check above and the server-side reception 
                        // of the request to retrieve terms. If so, the term list may
                        // appear truncated. This extra check verifies that we actually
                        // have got all the terms.
                        if (!list.isValid())
                        {
                            System.out.println("Term list #"+Integer.toString(listnr)+" is no longer valid!");
                        }
                    }
                }
                break;

            case 5:
                listnr = selectTermList(stdin);
                if (listnr > 0)
                {
                    listnr--;
                    TdbTermList list = termLists.get(listnr);
                    termLists.remove(listnr);
                    
                    // Remember to call close() on the term list so that
                    // server-side resources are released!
                    list.close();
                }
                break;
                
            case 6:
                boolean enable = !m_command.getUseOnDemandTermLists();
                m_command.setUseOnDemandTermLists(!m_command.getUseOnDemandTermLists());;
                System.out.println("On-demand term lists are now "+ (enable?"enabled":"disabled"));
                break;
                
            case 7:
                int seconds = m_command.getTermListTimeout();
                seconds = stdin.readint("Timeout (seconds)", seconds, true);
                m_command.setTermListTimeout(seconds);
                break;
                
        }

        return finished;
    }

    /**
     * Execute a CCL command, post-process it to show the results on the screen
     *
     * @param cmd The command to execute
     * @return true if the user did something signalling a desire to stop using
     * CCL
     */
    public boolean execCcl(String cmd)
    {
        TdbKernelWindow w;
        String dym;
        boolean finished = false;
        int i;

        // We get a null command for an empty line.
        if (null == cmd)
        {
            return false;
        }

        try
        {
            // Execute the CCL command typed in by the user
            m_command.resetMessages();
            m_command.execDirect(cmd);
            for (String msg : m_command.messageList())
            {
                System.out.println(msg);
            }

            // Post-process the command
            switch (m_command.getCommandType())
            {

                // Commands that seek to abort the CCL session (STOP)
                case Abort:
                    m_session.logout();
                    finished = true;
                    break;

                // Commands that update the search history in some way (BASE, FIND, FUZZ, DEFINE, LIST, etc.)
                case HistoryReplace:
                case HistoryUpdate:
                    for (TdbHistoryDetail detail : m_command.historyUpdates())
                    {
                        System.out.println("Search " + detail.getSearchId() + ": "
                                + detail.getRecordCount() + " records in database "
                                + m_command.getCurrentBase());
                    }
                    dym = m_command.getDidYouMean();
                    if (dym.length() > 0)
                    {
                        System.out.println("Did you mean: " + dym);
                    }
                    break;

                // Commands that produce lists of terms (DISPLAY)
                case TermList:
                {
                    TdbTermList list = m_command.getTermList();

                    if (list.isOnDemand())
                    {
                        termLists.add(list);
                        m_command.setTermList(new TdbTermList());
                        System.out.println("On-demand term list #" + Integer.toString(termLists.size()) 
                                + " with " + Integer.toString(list.size()) + " terms created.");
                    } 
                    else
                    {
                        i = 1;
                        System.out.println(Integer.toString(list.size()) + " terms");
                        for (TdbTerm term : list)
                        {
                           System.out.println("T=" + i++ + "\t<" + term.getRecordCount() + ">\t" + term.getTerm());
                        }
                    }
                    break;
                }

                // Commands that produce trees of terms (Display UP/DOWN/etc.)
                case TermTree:
                {
                    TdbTermTree tree = m_command.getTermTree();
                    System.out.println("" + tree.getRootCount() + " main terms.\n");
                    for (TdbTreeEntry entry : tree.roots())
                    {
                        printThesTerm(entry, 0);
                    }
                    break;
                }

                // Commands that produce output (SHOW, MORE, BACK, NEXT, PREV, etc.)
                case Output:
                    w = m_command.getAffectedWindow();
//				System.out.println(w.toFormattedString(">>", "<<", "\n"));
                    System.out.println(w.toString());
                    if (!w.isAtBottom())
                    {
                        System.out.println("Type More to continue");
                    }
                    break;
            }
        } 
        catch (TdbException e)
        {
            System.out.println(e.getDebugMessage());
        }

        return finished;
    }
    
    // Utility function used to recurse through a tree of thesaurus terms
    private static final String spaces = "                                                      ";

    private void printThesTerm(TdbTreeEntry entry, int indent)
    {
        System.out.println(spaces.substring(0, indent) + entry.getName());
        for (TdbTreeEntry child : entry.children())
        {
            printThesTerm(child, indent + 2);
        }
    }

    /**
     * Entry point for the program
     *
     * @param args Not used
     */
    public static void main(String[] args)
    {
        try
        {
            TermLists app = new TermLists();
            Console stdin = new Console();
            boolean done = false;

            app.setup();

            while (!done)
            {
                done = app.process(stdin);
            }
        } 
        catch (TdbException e)
        {
            System.out.println(e.getDebugMessage());
        }
        catch (Exception x)
        {
            x.printStackTrace(System.out);
            Throwable cause = x.getCause();
            while (cause != null)
            {
                System.out.println("CAUSED BY:");
                System.out.println(cause.toString());
                cause.printStackTrace(System.out);
                cause = cause.getCause();
            }
        }
    }
}
