///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2023 Smaser AG
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//      TF  Mar 2023, 2023    Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.data;

import java.util.ArrayList;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.TdbFieldType;
import com.tietoenator.trip.jxp.database.TdbDatabaseDesign;
import com.tietoenator.trip.jxp.database.TdbFieldDesign;
import com.tietoenator.trip.jxp.data.TdbRecord;
import com.tietoenator.trip.jxp.data.TdbField;
import com.tietoenator.trip.jxp.session.TdbSession;

import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;


/**
 * This class demonstrates how to efficiently store a very large number of 
 * values into a field.
 *
 * This sample assumes a database design without record names and with no
 * mandatory fields. It must have at least one TEXT or PHRASE field allowing
 * an unlimited max number of values.
 */
public class BulkPut
{
    private TdbSession m_session;
    private String m_database = null;
    private String m_textField = null;
	private int m_maxValues = 100000;
	private boolean m_optimized = true;

    /**
     * Constructor
     * @throws TdbException
     */
    public BulkPut()
            throws TdbException
    {
        // Create a session on a physical server (no grid support)
        m_session = SessionFactory.newInstance("**** TRIPjxp TdbField.AppendValues example ****", false);
    }

    void close()
    {
        if (m_session != null)
        {
            try { m_session.logout(); } catch (Exception ignore) {}
            m_session = null;
        }
    }

    private void setDatabase(String db)
    {
        m_database = db;
    }

    private void setTextField(String tf)
    {
        m_textField = tf;
    }
	
	private void setMaxValues(int maxValues)
	{
		m_maxValues = maxValues;
	}
	
	private void setOptimized(boolean optimized)
	{
		m_optimized = optimized;
	}

	private void printElapsedTime(long startTime)
	{
		long elapsedMs = System.currentTimeMillis() - startTime;
		long secTime = elapsedMs / 1000;
		long msecTime = elapsedMs - secTime * 1000;
		System.out.println("... "+Long.toString(secTime)+"."+String.format("%03d", msecTime)+" s");
	}

    /**
     * Perform text extraction or HTML conversion as specified.
     */
    public void process()
            throws Exception
    {
        TdbRecord rec = null;

        TdbDatabaseDesign db = new TdbDatabaseDesign(m_session);
		db.get(m_database);
		TdbFieldDesign fdes = db.getFieldByName(m_textField);
		if (fdes == null)
		{
			System.out.println("ERROR: Specified field does not exist: "+m_textField);
			return;
		}
		if (fdes.getTypeId() != TdbFieldType.TextField && fdes.getTypeId() != TdbFieldType.PhraseField)
		{
			System.out.println("ERROR: The field is not TEXT or PHRASE: "+m_textField);
			return;
		}
		
        rec = new TdbRecord(m_session, db, false);
		TdbField fld = rec.getHead().createField(fdes);
		
		System.out.println("Preparing data collection to store");
		
		long starttime = System.currentTimeMillis();
		String msg = "unde omnis iste natus error sit voluptatem accusantium";
		ArrayList<String> values = new ArrayList<String>(m_maxValues);
		for (int i=0; i<m_maxValues; i++)
			values.add(msg);
		printElapsedTime(starttime);
		
		System.out.println("Adding "+Integer.toString(m_maxValues)+" values to the record");
		starttime = System.currentTimeMillis();
		if (m_optimized)
			fld.appendValues(values);
         else
         {
            for (int i=0; i<m_maxValues; i++)
               fld.appendValue(msg);
         }
		printElapsedTime(starttime);

		System.out.println("Comitting the record");
		starttime = System.currentTimeMillis();
		rec.commit();
		printElapsedTime(starttime);

		System.out.println("Record "+Integer.toString(rec.getRecordId())+" comitted to the database");
    }

    /**
     * Main entry point
     * @param args Not used
     */
    public static void main(String[] args)
    {
        BulkPut program = null;
        try
        {
            program = new BulkPut();
            String db = null, txt = null, pname = null, pval = null;
            Console stdin = new Console();
			
			System.out.println("This sample assumes a database design without record names and with no");
			System.out.println("mandatory fields. It must have at least one head field of type TEXT or ");
			System.out.println("PHRASE that allows an unlimited max number of values.");
			System.out.println();

            // Get a valid database name, and set the list of
            // retrieval fields from it
            while (db == null)
                db = stdin.readln("Database to add a record to", null);
            program.setDatabase(db);

            while (txt == null)
                txt = stdin.readln("Text or phrase field", null);
            program.setTextField(txt);

			program.setMaxValues(stdin.readint("Nr of values to store", 100000, true));
			program.setOptimized(stdin.readbool("Optimized preparation", true, true));

            // Perform the operation
            program.process();
        }
        catch (TdbException e)
        {
            System.out.println();
            System.out.println("[ERROR]");
            System.out.println(e.getDebugMessage());
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            if (program != null)
            {
                program.close();
                program = null;
            }
        }
    }
}
