///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2019 infinIT Services GmbH
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		TF	Apr 11, 2019	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.data;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.data.TdbField;
import com.tietoenator.trip.jxp.data.TdbRecord;
import com.tietoenator.trip.jxp.data.TdbSearch;
import com.tietoenator.trip.jxp.data.TdbSearchSet;
import com.tietoenator.trip.jxp.data.TdbStructuredField;
import com.tietoenator.trip.jxp.data.TdbTextField;
import com.tietoenator.trip.jxp.data.TdbFieldTemplate;
import com.tietoenator.trip.jxp.data.TdbFieldFetchType;
import com.tietoenator.trip.jxp.database.TdbDatabaseDesign;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.session.TdbSession;
import java.util.Vector;

/**
 * Sample program for TRIPjxp that shows how to retrieve output format
 * reports as a kind of virtual field values using a retrieval template.
 */
public class FormattedValues
{
    private TdbSession m_session;
    private TdbDatabaseDesign m_db;
	private TdbSearch m_searchHandler;

    /**
     * Constructor
     * @throws TdbException
     */
    public FormattedValues()
            throws TdbException
    {
        // Create a session on a physical server (no grid support)
        m_session = SessionFactory.newInstance("**** FormattedValues example ****", false);
    }

    public void initialize(String database) throws TdbException
    {
		// Create our search handler
		m_searchHandler = new TdbSearch(m_session);

		// Load the database design once so that we don't have to do it 
		// repeatedly.
        m_db = new TdbDatabaseDesign(m_session);
        m_db.get(database);
    }


    public void search(String qry, int maxRecords)
            throws TdbException
    {
        int recordIdx = 0;
        TdbRecord template = null;
		TdbFieldTemplate virtualField;
		

		// Create field template and associate it with the database
		// we already loaded.
		template = new TdbRecord(m_session,m_db,false);
	
		// The regular fields we want
		template.addToTemplate("PERSON");
		template.addToTemplate("SPEAKER");
		
		// The output format we want applied to each record.
		// The name of the output format is the first argument. 
		// The third argument (size hint) is irrelevant for formatted values.
		virtualField = new TdbFieldTemplate("SHORT", 
		                                    TdbFieldFetchType.FormattedValue, 0);
		template.addToTemplate(virtualField);
		
		// Associate the template with the search handler
		m_searchHandler.setAutomaticRetrievalTemplate(false);
		m_searchHandler.setRetrievalTemplate(template);

        // Open the database
        m_searchHandler.execute("BASE " + m_db.getName());

		if (qry != null)
		{
			// Clear search history associated with this search handler.
			// Here it just removes the search set associated with the
			// BASE command.
			m_searchHandler.clear();

			// Execute the specified search.
			m_searchHandler.execute("FIND " + qry);
		}


        // Iterate through the found records.
        TdbSearchSet searchSet = m_searchHandler.getLastSearchSet();
        for (TdbRecord rec : searchSet)
        {
            System.out.print("\nRecord #");
            System.out.println(Integer.toString(rec.getRecordId()));

            // Display the retrieved data (head fields only in this example)
			// NB: Formatted values are always returned as head fields.
            for (TdbField f : rec.getHead().fields())
            {
                System.out.print(f.getName());
                System.out.print(": ");

                if (f instanceof TdbStructuredField)
                {
                    boolean bFirstSubfield = true;
                    for (String subfield : f.values())
                    {
                        if (bFirstSubfield)
                        {
                            bFirstSubfield = false;
                        }
                        else
                        {
                            System.out.print(", ");
                        }
                        System.out.print(subfield);
                    }
                    System.out.println();
                }
                else if (f instanceof TdbTextField)
                {
                    TdbTextField txtField = (TdbTextField)f;
                    System.out.println();
                    System.out.println(txtField.getText());
                }
            }

            System.out.println();
            if (++recordIdx >= maxRecords)
                break;
        }

        // Release server resources (including search sets) assoicated
        // with the TdbSearch object
        m_searchHandler.clear();
    }

    /**
     * Main entry point
     * @param args Not used
     */
    public static void main(String[] args)
    {
        try
        {
            FormattedValues app = new FormattedValues();

            Console stdin = new Console();
            String db = null, qry = null;
            int maxRecords = 0;

			// Perform one-time initialization
            app.initialize("ALICE");

            // Prompt for a query
			qry = stdin.readln("Query (omit \"FIND\" or \"FUZZ\" keyword)", null);
            if (null == qry)
				System.out.println("Using the database search set\n");
				
            // Prompt for max records to retrieve
            maxRecords = stdin.readint("Max records to retrieve (3)", 3, true);

            // Perform the retrieval
            app.search(qry, maxRecords);
        }
        catch (TdbException e)
        {
            System.out.println(e.getDebugMessage());
        }
    }
}
