///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2006-2009 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		GF	Jul 31, 2006	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.data;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.data.TdbComponent;
import com.tietoenator.trip.jxp.data.TdbField;
import com.tietoenator.trip.jxp.data.TdbFieldTemplate;
import com.tietoenator.trip.jxp.data.TdbRecord;
import com.tietoenator.trip.jxp.data.TdbRecordSet;
import com.tietoenator.trip.jxp.database.TdbDatabaseDesign;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.session.TdbSession;
import java.util.Iterator;

/**
 * This class demonstrates a simple means of placing a request to a server for
 * a set of records, optionally conforming to a specified query. The set of
 * records retrieved are displayed on the console.
 */
public class RecordSet
{

   private TdbSession m_session;

   /**
    * Constructor
    * @throws TdbException
    */
   public RecordSet()
           throws TdbException
   {
      // Create a session on a physical server (no grid support)
      m_session = SessionFactory.newInstance("**** Simple data retrieval example ****", false);
   }

   /**
    * Retrieve a range of records from the named database, optionally using
    * a query to subset the database, picking the record range as specified
    *
    * @param dbname Name of database from which to retrieve records
    * @param qry (optional) query
    * @param from First record # to be retrieved
    * @param to Last record # to be retrieved
    * @throws TdbException
    */
   public void getRecords(String dbname, String thesname, String qry, int from, int to, boolean reversed, boolean fieldSizeOnly)
           throws TdbException
   {
      TdbRecordSet rs = new TdbRecordSet(m_session);

      // Retrieve the database design
      TdbDatabaseDesign db = new TdbDatabaseDesign(m_session);
      db.get(dbname);

      // Set the general parameters for the retrieval
      rs.setDatabase(dbname);
      rs.setThesaurus(thesname);
      rs.setQuery(qry);
      rs.setFrom(from);
      rs.setTo(to);
      rs.setRetrieveReversed(reversed);

      // Create a "template" for the fields the result should contain
      TdbRecord tmpl = new TdbRecord(m_session, db, true);
      tmpl.setTemplateMarkup(">>", "<<");
      rs.setRetrievalTemplate(tmpl);

      // If we're only retrieveing field sizes, disable field retrieval
      // of field content for all fields.
      if (fieldSizeOnly)
      {
          Iterator<TdbFieldTemplate> fti = tmpl.getFieldTemplates();
          while (fti.hasNext())
              fti.next().setFieldContent(false);
      }

      // Retrieve the records from the server -- this is the only
      // method call that actually hits the network in any way
      rs.get();

      // Use the simplest method of coercing records to strings...
      for (TdbRecord record : rs.records())
      {
         // First the head record structure
         System.out.println("\nRecord " + record.getRecordId() + "\n");
         for (TdbField field : record.getHead().fields())
         {
            System.out.println(field.getName() +
                    " [size: " + Integer.toString(field.getFieldSize()) +
                    "]: " + field.toString());
         }

         // Then any part records
         for (TdbComponent part : record.parts())
         {
            System.out.println("\nPart " + part.getId());
            for (TdbField field : part.fields())
            {
                System.out.println(field.getName() +
                        " [size: " + Integer.toString(field.getFieldSize()) +
                        "]: " + field.toString());
            }
         }
      }

      // Terminate the session
      m_session.logout();
   }

   /**
    * Main entry point
    * @param args Not used
    */
   public static void main(String[] args)
   {
      try
      {
         RecordSet rs = new RecordSet();

         Console stdin = new Console();
         String db = null, qry = null;
         int from = 0, to = 0;
         boolean rev = false;
		 boolean fieldSizeOnly = false;
         String thes = null;

         // Get a valid database name, and set the list of
         // retrieval fields from it
         while (db == null)
         {
            db = stdin.readln("Database", null);
         }

         if (stdin.readint("Define thesaurus? (1|0)", 0, true) != 0)
         {
            while (thes == null)
                thes = stdin.readln("Thesaurus", "THESALI");
         }

         // Prompt for an optional query
         qry = stdin.readln("Query (optional)", null);

         // Prompt for the retrieval range
         from = stdin.readint("From (1)", 1, true);
         to = stdin.readint("To (10)", 10, true);

         // Prompt for reverse retrieval
         if (stdin.readint("Reverse retrieval? (1|0)", 0, true) != 0)
         {
            rev = true;
         }
		 
         // Prompt for reverse retrieval
         if (stdin.readint("Get field sizes only? (1|0)", 0, true) != 0)
         {
            fieldSizeOnly = true;
         }

         // Perform the retrieval
         rs.getRecords(db, thes, qry, from, to, rev, fieldSizeOnly);
      }
      catch (TdbException e)
      {
         System.out.println(e.getDebugMessage());
      }
   }
}
