///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2006-2009 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		GF	Aug 1, 2006	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.data;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;

import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.data.TdbDataFormat;
import com.tietoenator.trip.jxp.data.TdbFieldFetchType;
import com.tietoenator.trip.jxp.data.TdbFieldTemplate;
import com.tietoenator.trip.jxp.data.TdbRecord;
import com.tietoenator.trip.jxp.data.TdbRecordSet;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.session.TdbSession;

/**
 * This class demonstrates how results retrieved via a TdbRecordSet can be
 * transformed using the standard XSLT mechanisms within the JDK.
 */
public class Transform
{

   private Document m_result = null;

   /**
    * Retrieve the record set from the server
    * @param qry (option) Query to execute
    * @param from Low record # to retrieve
    * @param to High record # to retrieve
    * @throws TdbException
    */
   public void getRecords(String qry, int from, int to)
           throws TdbException
   {
      // Create a physical session, grid not supported
      TdbSession session = SessionFactory.newInstance("**** Transforming data retrieval example ****", false);
      TdbRecordSet rs = new TdbRecordSet(session);
	  TdbFieldFetchType textFetchMode = (qry != null) ? TdbFieldFetchType.FocusOrExtract : TdbFieldFetchType.EntireValue;

      // Set the general parameters for the retrieval
      rs.setDatabase("Alice");
      rs.setQuery(qry);
      rs.setFrom(from);
      rs.setTo(to);

      // Create a "template" for the fields the result should contain
      TdbRecord tmpl = new TdbRecord(session);
      tmpl.addToTemplate("Chapter");
      tmpl.addToTemplate("Speaker");
      tmpl.addToTemplate(new TdbFieldTemplate("Txt", textFetchMode, 0));
      rs.setRetrievalTemplate(tmpl);

      // Retrieve the records from the server as an XML document
      m_result = rs.get(TdbDataFormat.ELEMENT);

      // Terminate the session
      session.logout();
   }

   /**
    * Transform the retrieved record set into a String representation
    * @param html if set true, the stylesheet used will generate HTML, otherwise plain text
    * @return A String object containing the result of the transformation
    * @throws Exception
    */
   public String transform(boolean html)
           throws Exception
   {
      // Read the stylesheet from our JAR
      String name = "com/tietoenator/jxp/examples/data/" + (html ? "html.xsl" : "text.xsl");
      InputStream is = getClass().getClassLoader().getResourceAsStream(name);

      // Transform our result document into a String
      ByteArrayOutputStream os = new ByteArrayOutputStream();
      TransformerFactory.newInstance().newTransformer(new StreamSource(is)).transform(new DOMSource(m_result), new StreamResult(os));
      is.close();
      return os.toString();
   }

   /**
    * Main entry point
    * @param args Not used
    */
   public static void main(String[] args)
   {
      try
      {
         Transform tx = new Transform();
         Console stdin = new Console();

         // Prompt for an optional query
         String qry = stdin.readln("Query (optional)", null);

         // Prompt for transformation type
         String type = stdin.readln("Transformation (text, html)", "text");

         // Perform the retrieval, first ten records of the database or the search result
         tx.getRecords(qry, 1, 10);

         // Transform the result from XML to either text or HTML
         System.out.println(tx.transform(type.equalsIgnoreCase("html")));
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }
   }
}
